import java.awt.*;
import java.awt.image.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;

/**
 * ImgGetr is a utility class to get images.  It is
 * similar and simpler than MediaTracker.  The one
 * advantage of ImgGetr is that if an error occurs
 * getting the image (such as the url or file specified
 * does not exist) an image is created which can be used
 * to display an error message indicating to the user that
 * the image was not "gotten".
 * 
 * @author      John Pew
 */
public class ImgGetr {
    Image       image;          // The image
    String      image_file;     // The name of the image file
    int         image_width;    // Width of the image
    int         image_height;   // Height of the image
    URL         complete_url;
    InputStream in;
    String      errormsg;
    boolean     image_load;
    Applet      applet;

    /**
     * Construct an ImageGetter object.
     *
     * @param a The applet that has created this object
     */
    public ImgGetr(Applet a) {
        applet = a;
    }

    /**
     * Process the image. Check the usr and file and make sure that
     * we can actually load this thing. If not, then call the
     * createErrorMessageImage method to return an image with
     * the error message displayed in it.
     *
     * @param url The URL to load
     * @param file The file to load
     * @param loadnow If true, wait for the file to load before returning
     */
    public Image process(URL url, String file, boolean loadnow) {
        image_file = file;
        image_load = loadnow;
        int i;

        // See if the user specified an Image parameter
        // If not, then notify
        if(image_file == null) {
            String errormsg[] = { "Missing \"Image\" parameter!!" };
            return createErrorMessageImage(errormsg);
        }

        // If the user did specify an Image parameter
        if(image_file != null) {
            
            // First, check the URL and makes sure it's valid
            // We can't just do a getImage() on the specified
            // URL because getImage() returns immediately
            // and if the user has specified an invalid URL
            // (file) we don't get notified.
            try {
                complete_url = new URL(url, image_file);
            } catch(MalformedURLException e) {
                // It's unlikely that this execption will ever be
                // thrown because the calling class is
                // probably using getCodeBase()
                String errormsg[] = {
                        "Invalid \"Image\" value!!",
                        "MalformedURLException" };
                return createErrorMessageImage(errormsg);
            }
            if(complete_url != null) {
                image = applet.getImage(complete_url);
                if(image == null) {
                    String errormsg[] = { "getImage failed"};
                    System.out.println(errormsg[0]);
                    return createErrorMessageImage(errormsg);
                }
                if(loadnow)
                    loadImageAndWait(image, applet);
            }
        }
        return image;
    }

    /**
     * Checks to see if the specified image is actually 
     * prepared (loaded) and ready to display
     * Return true if loaded, otherwise false
     *
     * @param image the image to check
     * @param applet the applet
     */
    public boolean isImagePrepared(Image image, Applet applet) {
        boolean ImagePrepared;
        ImagePrepared = applet.prepareImage(image, applet);
        return ImagePrepared;
    }

    /**
     * Begins the preparation (loading) of the image
     * This function returns immediately
     * The image is loaded in a thread
     *
     * @param image the image to prepare
     * @param applet the applet
     */
    public void prepareImage(Image image, Applet applet) {
        boolean ImagePrepared;
        ImagePrepared = applet.prepareImage(image, applet);
    }

    /**
     * Prepares (loads) the image and does not return 
     * until the loading is complete
     *
     * @param image the image to load
     * @param applet the applet
     */
    public synchronized void loadImageAndWait(Image image, Applet applet) {
        int checkImageFlags;
        boolean ImagePrepared;

        ImagePrepared = applet.prepareImage(image, applet);
        if(ImagePrepared == false) {
            while(((checkImageFlags = 
                    applet.checkImage(image, applet)) &
                            ImageObserver.ALLBITS) == 0) {
                try {
                    wait(100);
                } catch (InterruptedException e){}
            }
        }
    }

    /**
     * Create an error message as an image so that
     * the applet can display the error message where
     * the image would have otherwise been displayed
     *
     * @param e An array of error messages
     */
    public Image createErrorMessageImage(String e[]) {
        Image errorImage;
        Graphics errorGC;
        Font font;
        Font errorfont;
        FontMetrics fontmetrics;
        int height;
        int position = 25;
        int i;

        errorImage = applet.createImage(applet.size().width, applet.size().height);
        errorGC = errorImage.getGraphics();
        errorGC.setColor(applet.getBackground());
        errorGC.fillRect(0, 0, applet.size().width, applet.size().height);
        errorGC.setColor(applet.getForeground());
        font = applet.getFont();
        errorfont = new Font("Dialog", Font.BOLD, 12);
        applet.setFont(errorfont);
        fontmetrics = applet.getFontMetrics(errorfont);
        height = fontmetrics.getHeight();

        errorGC.setFont(new Font(errorfont.getName(),
                                Font.BOLD, errorfont.getSize()+4));
        errorGC.drawString("ERROR:", 10, position);
        errorGC.setFont(errorfont);
        for(i = 0;i<e.length;i++) {
            if(e[i] != null) {
                position += height;
                errorGC.drawString(e[i], 25, position);
            }
        }
        applet.setFont(font);
        return errorImage;
    }

    public synchronized int getWidth(Image image, ImageObserver observer) {
        int width;
        while((width = image.getWidth(observer)) == -1) {
            try {wait(100);} catch (InterruptedException e){}
        }
        return width;
    }

    public synchronized int getHeight(Image image, ImageObserver observer) {
        int height;
        while((height = image.getHeight(observer)) == -1) {
            try {wait(100);} catch (InterruptedException e){}
        }
        return height;
    }

    public int[] getPixels(Image image, ImageObserver observer) {
        int pixels[];
        int w, h;
        PixGrab tmp_pg;
        boolean retval;

        w = getWidth(image, observer);
        h = getHeight(image, observer);
        loadImageAndWait(image, applet);
        pixels = new int[w * h];
        tmp_pg = new PixGrab(image, 0, 0, w, h, pixels, 0, w);
        try {
            retval = tmp_pg.grabPixels(0);
        } catch (InterruptedException e) {
            System.err.println("Interrupted waiting for pixels");
        }
        return pixels;
    }

    public int[] getPixels(Image image, Applet applet) {
        int pixels[];
        int w, h;
        PixGrab tmp_pg;
        boolean retval;

        w = getWidth(image, applet);
        h = getHeight(image, applet);
        loadImageAndWait(image, applet);
        pixels = new int[w * h];
        tmp_pg = new PixGrab(image, 0, 0, w, h, pixels, 0, w);
        try {
            retval = tmp_pg.grabPixels(0);
        } catch (InterruptedException e) {
            System.err.println("Interrupted waiting for pixels");
        }
        return pixels;
    }


    /*
     * This is an alternative way of grabbing the pixels.
     * It may be necessary to use this version if the PixelGrabber
     * continues to cause fatal errors
     */

    /*
    public static int[] getPixels(Image image,
                                  Applet applet) {
        int w, h;
        int pixels[];
        Image tmpimage, tmpimage2;
        Graphics tmpGC;

        w = getWidth(image, applet);
        h = getHeight(image, applet);
        loadImageAndWait(image, applet);
        CatchFlt catchFilter = new CatchFlt();
        catchFilter.setsize(applet, w, h);
        tmpimage = applet.createImage(new FilteredImageSource(image.getSource(),
                                                    catchFilter));
        tmpimage2 = applet.createImage(w, h);
        tmpGC = tmpimage2.getGraphics();
        tmpGC.drawImage(tmpimage, 0, 0, applet);
        loadImageAndWait(tmpimage2, applet);
        pixels = catchFilter.getPixels();
        return pixels;
    }
    */
}

/*
class CatchFlt extends RGBImageFilter {
    int index;
    int w, h;
    int pixels[];
    Applet applet;
    int mul;
    Color fg, bg;

    public CatchFlt() {
        // The filter's operation does not depend on the
        // pixel's location, so IndexColorModels can be
        // filtered directly.
        canFilterIndexColorModel = false;
    }
    public CatchFlt(Applet applet, int wid, int hei) {
        this.applet = applet;
        // The two sizes may not exactly be equal proportional, so
        // choose the bigger
        w = wid;
        h = hei;
        pixels = new int[h*w];
        canFilterIndexColorModel = false;
    }

    public void setsize(Applet applet, int wid, int hei) {
        this.applet = applet;
        // The two sizes may not exactly be equal proportional, so
        // choose the bigger
        w = wid;
        h = hei;
        pixels = new int[h*w];
        return;
    }

    public int filterRGB(int x, int y, int rgb) {

        pixels[(y*w)+x] = rgb;

        return rgb;
    }

    public int[] getPixels() {
        return pixels;
    }
}
*/
